# This program (based on calibration_ICP_LL.R) is for producing 20%x20% calibration pictures for CICP
# Used in the paper for Figure 6

start <- Sys.time()	# timing: start

# preparing the data
N <- 4601   # size of the data set
N.training <- 2602
N.calibration <- 999
N.test <- 1000
original.data <- read.csv("spam.data")

N.colours <- 8   # 8 in the paper
N.bins <- 211    # number of bins (prime number)
eps <- seq(from=0,to=0.2,length.out=N.bins+1)  # significance levels to consider
err <- matrix(0.0,nrow=N.colours,ncol=N.bins+1)       # percentage of errors at each significance level
err.email <- matrix(0.0,nrow=N.colours,ncol=N.bins+1) # the same for email only
err.spam <- matrix(0.0,nrow=N.colours,ncol=N.bins+1)  # the same for spam only

for (colour in 1:N.colours) {
  set.seed(colour-1)             # reproducibility
  permutation <- sample(1:N)
  permuted.data <- original.data[permutation,]
  rownames(permuted.data) <- 1:N
  training.set <- permuted.data[1:N.training,]
  calibration.set <- permuted.data[(N.training+1):(N.training+N.calibration),]
  rownames(calibration.set) <- 1:N.calibration
  test.set <- permuted.data[(N.training+N.calibration+1):(N.training+N.calibration+N.test),]
  rownames(test.set) <- 1:N.test

  # fit the model
  gbm1 <- gbm(V58~V1+V2+V3+V4+V5+V6+V7+V8+V9+V10+V11+V12+V13+V14+V15+V16+V17+V18+V19+V20+V21+V22+V23+V24+V25+V26+V27+V28+V29+V30+V31+V32+V33+V34+V35+V36+V37+V38+V39+V40+V41+V42+V43+V44+V45+V46+V47+V48+V49+V50+V51+V52+V53+V54+V55+V56+V57,
      data=training.set,
      distribution="bernoulli",    # adaboost or bernoulli
      n.trees=4000,                # number of trees
      shrinkage=0.005,             # shrinkage or learning rate
      interaction.depth=3,         # 1: additive model, 2: two-way interactions, etc.
      bag.fraction = 0.5,          # subsampling fraction, 0.5 is probably best
      train.fraction = 1.0,        # all data used for training
      n.minobsinnode = 10,         # minimum total weight needed in each node
      cv.folds = 5,                # do 5-fold cross-validation
      keep.data=TRUE)              # keep a copy of the dataset with the object

  # check performance using 5-fold cross-validation
  best.iter <- gbm.perf(gbm1,method="cv")
  print(best.iter)

  # predict on the new data using "best" number of trees
  # .predict will be on the logit scale
  calibration.predict <- predict.gbm(gbm1,calibration.set,best.iter)
  test.predict <- predict.gbm(gbm1,test.set,best.iter)

  # conformity scores
  calibration.scores <- (2*calibration.set$V58-1)*calibration.predict
  test.scores <- (2*test.set$V58-1)*test.predict

  labels <- test.set$V58  # vector of labels for the test set
  calibration.labels <- calibration.set$V58  # vector of labels for the calibration set

  N.email <- 0  # number of email in the test set
  N.spam <- 0   # number of spam in the test set

  # For the first plot:
  pv <- rep.int(0,N.test)   # p-values for all test examples

  for (i in 1:N.test) {
    rank.actual <- sum(as.numeric(calibration.scores[calibration.labels==labels[i]] <= test.scores[i]))
    N.calibration.actual <- sum(as.numeric(calibration.labels==labels[i]))
    pv[i] <- (rank.actual+1)/(N.calibration.actual+1)
    if (labels[i]==0) {N.email <- N.email+1}  # counting email
    if (labels[i]==1) {N.spam <- N.spam+1}    # counting spam
  }

  # For the two other plots:
  pv.email <- pv[labels==0]   # p-values for email
  pv.spam <- pv[labels==1]    # p-values for spam

  for (e in 1:N.bins+1) {  # loop over significance levels
    for (i in 1:N.test) {  # loop over test examples
      if (pv[i] <= 0.2*(e-1)/N.bins) {  # if an error is made
        err[colour,e] <- err[colour,e] + 1/N.test
      }
    }
  }

  for (e in 1:N.bins+1) {  # loop over significance levels
    for (i in 1:N.email) {  # loop over email
      if (pv.email[i] <= 0.2*(e-1)/N.bins) {  # if an error is made
        err.email[colour,e] <- err.email[colour,e] + 1/N.email
      }
    }
  }

  for (e in 1:N.bins+1) {  # loop over significance levels
    for (i in 1:N.spam) {  # loop over spam
      if (pv.spam[i] <= 0.2*(e-1)/N.bins) {  # if an error is made
        err.spam[colour,e] <- err.spam[colour,e] + 1/N.spam
      }
    }
  }
} # end of big for

par(mfrow=c(1,3))       # I want 3 plots in one

# par(mar=c(2,2,1,1))   # smaller margins (the default, (6,4,4,2), is too big unless I need labels for axes and a title)

# Overall calibration plot
for (colour in 1:N.colours) {
  if (colour==1) {
    plot(eps,err[colour,],col=colour,type="l",lty="solid",xlim=c(0,0.2),ylim=c(0,0.2),
      xlab="significance level",ylab="error rate",main="Overall calibration plot")
  }
  if (colour!=1) {
    lines(eps,err[colour,],col=colour,type="l",lty="solid")
  }
} # end of for

# Calibration plot for email
for (colour in 1:N.colours) {
  if (colour==1) {
    plot(eps,err.email[colour,],col=colour,type="l",lty="solid",xlim=c(0,0.2),ylim=c(0,0.2),
      xlab="significance level",ylab="error rate",main="Calibration plot for email")
  }
  if (colour!=1) {
    lines(eps,err.email[colour,],col=colour,type="l",lty="solid")
  }
} # end of for

# Calibration plot for spam
for (colour in 1:N.colours) {
  if (colour==1) {
    plot(eps,err.spam[colour,],col=colour,type="l",lty="solid",xlim=c(0,0.2),ylim=c(0,0.2),
      xlab="significance level",ylab="error rate",main="Calibration plot for spam")
  }
  if (colour!=1) {
    lines(eps,err.spam[colour,],col=colour,type="l",lty="solid")
  }
} # end of for

end <- Sys.time()	# timing: end
cat("Finished. Time elapsed:", difftime(end,start), "\n")
