# Used in the paper for Figures 2 and 3
# (their label conditional panels).
# This program is based on scatter_ICP.R

start <- Sys.time()	# timing: start
set.seed(0)             # to ensure reproducibility

# preparing the data

N <- 4601   # size of the data set
N.training <- 2602
N.calibration <- 999
N.test <- 1000
original.data <- read.csv("spam.data")
permutation <- sample(1:N)
permuted.data <- original.data[permutation,]
rownames(permuted.data) <- 1:N
training.set <- permuted.data[1:N.training,]
calibration.set <- permuted.data[(N.training+1):(N.training+N.calibration),]
rownames(calibration.set) <- 1:N.calibration
test.set <- permuted.data[(N.training+N.calibration+1):(N.training+N.calibration+N.test),]
rownames(test.set) <- 1:N.test

end <- Sys.time()	# timing: end 1
cat("Data read. Time elapsed:", difftime(end,start), "\n")

# fit the model
gbm1 <- gbm(V58~V1+V2+V3+V4+V5+V6+V7+V8+V9+V10+V11+V12+V13+V14+V15+V16+V17+V18+V19+V20+V21+V22+V23+V24+V25+V26+V27+V28+V29+V30+V31+V32+V33+V34+V35+V36+V37+V38+V39+V40+V41+V42+V43+V44+V45+V46+V47+V48+V49+V50+V51+V52+V53+V54+V55+V56+V57,
    data=training.set,
    distribution="bernoulli",    # adaboost or bernoulli
    n.trees=4000,                # number of trees
    shrinkage=0.005,             # shrinkage or learning rate,
                                 # 0.001 to 0.1 usually work
    interaction.depth=3,         # 1: additive model, 2: two-way interactions, etc.
    bag.fraction = 0.5,          # subsampling fraction, 0.5 is probably best
    train.fraction = 1.0,        # all data used for training
    n.minobsinnode = 10,         # minimum total weight needed in each node
    cv.folds = 5,                # do 5-fold cross-validation
    keep.data=TRUE)              # keep a copy of the dataset with the object

end <- Sys.time()	# timing: end 2
cat("Model fitted. Time elapsed:", difftime(end,start), "\n")

# check performance using an out-of-bag estimator
# OOB underestimates the optimal number of iterations
#best.iter <- gbm.perf(gbm1,method="OOB")
#print(best.iter)

# check performance using 5-fold cross-validation
best.iter <- gbm.perf(gbm1,method="cv")
print(best.iter)

# predict on the new data using "best" number of trees
# .predict will be on the logit scale
calibration.predict <- predict.gbm(gbm1,calibration.set,best.iter)
test.predict <- predict.gbm(gbm1,test.set,best.iter)

# conformity scores
calibration.scores <- (2*calibration.set$V58-1)*calibration.predict
test.scores <- (2*test.set$V58-1)*test.predict
test.scores.spam <- 1*test.predict
test.scores.email <- (-1)*test.predict

labels <- test.set$V58                     # vector of labels for the test set
calibration.labels <- calibration.set$V58  # vector of labels for the calibration set

# For the scatterplot:
p.email <- rep.int(0,N.test)   # p-values for email
p.spam <- rep.int(0,N.test)    # p-values for spam

for (i in 1:N.test) {
  rank.actual <- sum(as.numeric(calibration.scores[calibration.labels==labels[i]] <= test.scores[i]))
  rank.email <- sum(as.numeric(calibration.scores[calibration.labels==0] <= test.scores.email[i]))
  rank.spam <- sum(as.numeric(calibration.scores[calibration.labels==1] <= test.scores.spam[i]))
  N.email <- sum(as.numeric(calibration.labels==0))
  N.spam <- sum(as.numeric(calibration.labels==1))
  p.email[i] <- (rank.email+1)/(N.email+1)
  p.spam[i] <- (rank.spam+1)/(N.spam+1)
}

# Scatterplot of the p-values
# par(mar=c(2,2,1,1))   # smaller margins (the default, (6,4,4,2), is too big unless I need labels for axes)
par(mfrow=c(1,3))                  # I want 3 plots in one

# plot(p.email[labels==1],p.spam[labels==1],col="red",pch=4,asp=1,
#   xlim=c(0,1),ylim=c(0,1),xlab="email p-value",ylab="spam p-value",main="Email and spam")  # linear scale

plot(p.email[labels==1],p.spam[labels==1],col="red",pch=4,asp=1,
  xlim=c(0.001,1),ylim=c(0.001,1),xlab="email p-value",ylab="spam p-value",main="Email and spam",
  xaxt="n",yaxt="n",log="xy")    # log scale
axis(1, c(0.001,0.01,0.1,1))     # log scale
axis(2, c(0.001,0.01,0.1,1))     # log scale

points(p.email[labels==0],p.spam[labels==0],col="green",pch=1)

# plot(p.email[labels==0],p.spam[labels==0],col="green",pch=1,asp=1,
#   xlim=c(0,1),ylim=c(0,1),xlab="email p-value",ylab="spam p-value",main="Email only")  # linear scale
# plot(p.email[labels==1],p.spam[labels==1],col="red",pch=4,asp=1,
#   xlim=c(0,1),ylim=c(0,1),xlab="email p-value",ylab="spam p-value",main="Spam only")   # linear scale

plot(p.email[labels==0],p.spam[labels==0],col="green",pch=1,asp=1,
  xlim=c(0.001,1),ylim=c(0.001,1),xlab="email p-value",ylab="spam p-value",main="Email only",
  xaxt="n",yaxt="n",log="xy")   # log scale
axis(1, c(0.001,0.01,0.1,1))     # log scale
axis(2, c(0.001,0.01,0.1,1))     # log scale
plot(p.email[labels==1],p.spam[labels==1],col="red",pch=4,asp=1,
  xlim=c(0.001,1),ylim=c(0.001,1),xlab="email p-value",ylab="spam p-value",main="Spam only",
  xaxt="n",yaxt="n",log="xy")   # log scale
axis(1, c(0.001,0.01,0.1,1))     # log scale
axis(2, c(0.001,0.01,0.1,1))     # log scale

end <- Sys.time()	# timing: end
cat("Finished. Time elapsed:", difftime(end,start), "\n")
